@@ -4,9 +4,10 @@ import awkward as ak
 import numpy as np
 import pandas as pd
 import warnings
-
 import epix
 
+from .common import ak_num, calc_dt, apply_time_offset
+
 
 def main(args, return_df=False, return_wfsim_instructions=False, strax=False):
     """Call this function from the run_epix script"""
@@ -19,14 +20,16 @@ def main(args, return_df=False, return_wfsim_instructions=False, strax=False):
         tnow = starttime
 
     # Loading data:
-    inter, n_simulated_events = epix.loader(args['path'],
-                                            args['file_name'],
-                                            args['debug'],
-                                            outer_cylinder=args['outer_cylinder'],
-                                            kwargs_uproot_arrays={'entry_start': args['entry_start'],
-                                                                  'entry_stop': args['entry_stop']},
-                                            cut_by_eventid=args['cut_by_eventid']
-                                            )
+    
+    epix_file_loader = epix.file_loader(args['path'],
+                                        args['file_name'],
+                                        args['debug'],
+                                        outer_cylinder=args['outer_cylinder'],
+                                        kwargs={'entry_start': args['entry_start'],
+                                                'entry_stop': args['entry_stop']},
+                                        cut_by_eventid=args['cut_by_eventid'],
+                                        )
+    inter, n_simulated_events = epix_file_loader.load_file()
 
     if args['debug']:
         tnow = monitor_time(tnow, 'load data.')
@@ -50,7 +53,7 @@ def main(args, return_df=False, return_wfsim_instructions=False, strax=False):
     # Sort detector volumes and keep interactions in selected ones:
     if args['debug']:
         print('Removing clusters not in volumes:', *[v.name for v in args['detector_config']])
-        print(f'Number of clusters before: {np.sum(ak.num(result["ed"]))}')
+        print(f'Number of clusters before: {np.sum(ak_num(result["ed"]))}')
 
     # Returns all interactions which are inside in one of the volumes,
     # Checks for volume overlap, assigns Xe density and create_S2 to
@@ -65,29 +68,15 @@ def main(args, return_df=False, return_wfsim_instructions=False, strax=False):
     result = result[m]
 
     # Removing now empty events as a result of the selection above:
-    m = ak.num(result['ed']) > 0
+    m = ak_num(result['ed']) > 0
     result = result[m]
 
     if args['debug']:
-        print(f'Number of clusters after: {np.sum(ak.num(result["ed"]))}')
+        print(f'Number of clusters after: {np.sum(ak_num(result["ed"]))}')
         print('Assigning electric field to clusters')
 
-    if not ak.any(m):
-        # There are not any events left so return empty array:
-        #TODO: Add option for WFSim
-        warnings.warn('No interactions left, return empty DataFrame.')
-        if return_df:
-            if args.output_path and not os.path.isdir(args.output_path):
-                os.makedirs(args.output_path)
-
-            output_path_and_name = os.path.join(args.output_path,
-                                                args['file_name'][:-5] + "_wfsim_instructions.csv")
-            df = pd.DataFrame()
-            df.to_csv(output_path_and_name, index=False)
-            return
-
     # Add electric field to array:
-    efields = np.zeros(np.sum(ak.num(result)), np.float32)
+    efields = np.zeros(np.sum(ak_num(result)), np.float32)
     # Loop over volume and assign values:
     for volume in args['detector_config']:
         if isinstance(volume.electric_field, (float, int)):
@@ -100,28 +89,33 @@ def main(args, return_df=False, return_wfsim_instructions=False, strax=False):
                                             epix.awkward_to_flat_numpy(result.z)
                                             )
 
-    result['e_field'] = epix.reshape_awkward(efields, ak.num(result))
+    result['e_field'] = epix.reshape_awkward(efields, ak_num(result))
 
     # Sort entries (in an event) by in time, then chop all delayed
     # events which are too far away from the rest.
     # (This is a requirement of WFSim)
     result = result[ak.argsort(result['t'])]
-    dt = result['t'] - result['t'][:, 0]
+    dt = calc_dt(result)
     result = result[dt <= args['max_delay']]
 
     if args['debug']:
         print('Generating photons and electrons for events')
     # Generate quanta:
-    photons, electrons, excitons = epix.quanta_from_NEST(epix.awkward_to_flat_numpy(result['ed']),
-                                                         epix.awkward_to_flat_numpy(result['nestid']),
-                                                         epix.awkward_to_flat_numpy(result['e_field']),
-                                                         epix.awkward_to_flat_numpy(result['A']),
-                                                         epix.awkward_to_flat_numpy(result['Z']),
-                                                         epix.awkward_to_flat_numpy(result['create_S2']),
-                                                         density=epix.awkward_to_flat_numpy(result['xe_density']))
-    result['photons'] = epix.reshape_awkward(photons, ak.num(result['ed']))
-    result['electrons'] = epix.reshape_awkward(electrons, ak.num(result['ed']))
-    result['excitons'] = epix.reshape_awkward(excitons, ak.num(result['ed']))
+    if len(result) > 0:
+        photons, electrons, excitons = epix.quanta_from_NEST(epix.awkward_to_flat_numpy(result['ed']),
+                                                             epix.awkward_to_flat_numpy(result['nestid']),
+                                                             epix.awkward_to_flat_numpy(result['e_field']),
+                                                             epix.awkward_to_flat_numpy(result['A']),
+                                                             epix.awkward_to_flat_numpy(result['Z']),
+                                                             epix.awkward_to_flat_numpy(result['create_S2']),
+                                                             density=epix.awkward_to_flat_numpy(result['xe_density']))
+        result['photons'] = epix.reshape_awkward(photons, ak_num(result['ed']))
+        result['electrons'] = epix.reshape_awkward(electrons, ak_num(result['ed']))
+        result['excitons'] = epix.reshape_awkward(excitons, ak_num(result['ed']))
+    else:
+        result['photons'] = np.empty(0)
+        result['electrons'] = np.empty(0)
+        result['excitons'] = np.empty(0)
 
     if args['debug']:
         _ = monitor_time(tnow, 'get quanta.')
@@ -130,7 +124,7 @@ def main(args, return_df=False, return_wfsim_instructions=False, strax=False):
     number_of_events = len(result["t"])
     if args['source_rate'] == -1:
         # Only needed for a clean separation:
-        result['t'] = result['t'] - result['t'][:, 0]
+        result['t'] = calc_dt(result)
 
         dt = epix.times_for_clean_separation(number_of_events, args['max_delay'])
         if args['debug']:
@@ -150,9 +144,11 @@ def main(args, return_df=False, return_wfsim_instructions=False, strax=False):
         if args['debug']:
             print(f"Fixed event rate of {args['source_rate']} Hz")
 
-    result['t'] = result['t'][:, :] + dt
+    result['t'] = apply_time_offset(result, dt)
 
     # Reshape instructions:
+    if args['debug'] & (len(result) == 0):
+        warnings.warn('No interactions left, return empty DataFrame.')
     instructions = epix.awkward_to_wfsim_row_style(result)
     if args['source_rate'] != 0:
         # Only sort by time again if source rates were applied, otherwise
